$:.unshift(File.dirname(__FILE__)) unless $:.include?(File.dirname(__FILE__)) || $:.include?(File.expand_path(File.dirname(__FILE__)))
$:.unshift "#{File.expand_path(File.dirname(__FILE__))}/tamf/"
$:.unshift "#{File.expand_path(File.dirname(__FILE__))}/amf/types/"

require 'date'
require 'stringio'
require 'amf/common/hash_with_type'
require 'amf/pure/errors/all_files'
require 'amf/pure/helpers/all_files'
require 'amf/pure/mapping/class_mapper'


#todo: implement C version
# begin
#   require 'rocketamf/ext'
# rescue LoadError

require 'amf/pure'
# end

# AMF is a full featured AMF3 serializer/deserializer with support for
# bi-directional other language to ruby class mapping, custom serialization and mapping,
# remoting gateway helpers that follow AMF3 messaging specs, and a suite of specs
# to ensure adherence to the specification documents put out by Adobe. If the C
# components compile, then RocketAMF automatically takes advantage of them to
# provide a substantial performance benefit. In addition, RocketAMF is fully
# compatible with Ruby 2.0, 2.1.
#
# == Performance
#   # 100.000 objects
#   # Ruby 2.0
#   Testing pure AMF3:
#     minimum serialize time: 49.294496s
#     minimum deserialize time: 6.600238s
#
# == Example
#
#
# test_object =
#   {
#      first_name: "Greg",
#      last_name:  "House"
#   }
#
# data            = AMF::Root.serialize(test_object)
#
# restored_object = AMF::Root.serialize(data)
#
# == Mapping Classes Between Other language and Ruby
#
# RocketAMF provides a simple class mapping tool to facilitate serialization and
# deserialization of typed objects. Refer to the documentation of
# <tt>RocketAMF::ClassMapping</tt> for more details. If the provided class
# mapping tool is not sufficient for your needs, you also have the option to
# replace it with a class mapper of your own devising that matches the documented
# API.
#
# == Advanced Serialization (encode_amf and IExternalizable)
#
# RocketAMF provides some additional functionality to support advanced
# serialization techniques. If you define an <tt>encode_amf</tt> method on your
# object, it will get called during serialization. It is passed a single argument,
# the serializer, and it can use the serializer stream, the <tt>serialize</tt>
# method, the <tt>write_array</tt> method, the <tt>write_object</tt> method, and
# the serializer version. Below is a simple example that uses <tt>write_object</tt>
# to customize the property hash that is used for serialization.
#
# Example:
#
#   class TestObject
#     def encode_amf serializer
#       serializer.write_object self, @attributes
#     end
#   end
#
# If you plan on using the <tt>serialize</tt> method, make sure to pass in the
# current serializer version, or you could create a message that cannot be deserialized.
#
# Example:
#
#   class VariableObject
#     def encode_amf serializer
#         serializer.serialize(false)
#     end
#   end
#
module AMF

  class Root
    # Deserialize the AMF string _source_ of the given AMF version into a Ruby
    # data structure and return it. Creates an instance of <tt>AMF::Deserializer</tt>
    # with a new instance of <tt>AMF::Root.class_mapper</tt> and calls deserialize
    # on it with the given source, returning the result.
    public
    def self.deserialize(source)
      deserializer = AMF::Deserializer.new(@class_mapper.new)
      deserializer.deserialize(source)
    end

    # Serialize the given Ruby data structure _obj_ into an AMF3 stream.
    # Creates an instance of <tt>AMF::Serializer</tt>
    # with a new instance of <tt>AMF::Root.class_mapper</tt> and calls serialize
    # on it with the given object, returning the result.
    public
    def self.serialize(obj)
      serializer = AMF::Serializer.new(@class_mapper.new)
      serializer.serialize(obj)
    end

    public
    def self.register_class_alias(class_name_local, class_name_remote)
      @class_mapper.register_class_alias(class_name_local, class_name_remote)
    end

    public
    def self.register_classes(map)
      @class_mapper.register_classes(map)
    end

    public
    def self.clear_class_aliases
      @class_mapper.reset
    end

    public
    def self.class_mapper
      @class_mapper
    end

    private
    def self.init_class_mapper
      #todo: use c version
      # Activating the C Class Mapper:
      #   require 'rubygems'
      #   require 'amf'
      #   AMF::ClassMapper = AMF::Ext::FastClassMapping

      @class_mapper = AMF::ClassMapper
    end

    init_class_mapper

  end

end
